@@ -1,6 +1,6 @@
 /********************************************
 bi_funct.c
-copyright 2008-2021,2023, Thomas E. Dickey
+copyright 2008-2023,2024, Thomas E. Dickey
 copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,9 +11,16 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: bi_funct.c,v 1.125 2023/11/26 10:00:47 tom Exp $
+ * $MawkId: bi_funct.c,v 1.134 2024/09/05 17:44:48 tom Exp $
  */
 
+#define Visible_ARRAY
+#define Visible_BI_REC
+#define Visible_CELL
+#define Visible_RE_DATA
+#define Visible_STRING
+#define Visible_SYMTAB
+
 #include <mawk.h>
 #include <bi_funct.h>
 #include <bi_vars.h>
@@ -23,7 +30,6 @@ the GNU General Public License, version 2, 1991.
 #include <fin.h>
 #include <field.h>
 #include <regexp.h>
-#include <repl.h>
 
 #include <ctype.h>
 #include <math.h>
@@ -56,7 +62,7 @@ const BI_REC bi_funct[] =
 
    { "index",    bi_index,    2, 2 },
    { "substr",   bi_substr,   2, 3 },
-   { "sprintf",  bi_sprintf,  1, 255 },
+   { "sprintf",  bi_sprintf,  1, MAX_ARGS },
    { "sin",      bi_sin,      1, 1 },
    { "cos",      bi_cos,      1, 1 },
    { "atan2",    bi_atan2,    2, 2 },
@@ -81,7 +87,7 @@ const BI_REC bi_funct[] =
    { "strftime", bi_strftime, 0, 3 },
 #endif
 
-   { (char *)    0, (PF_CP) 0, 0, 0 }
+   { "",         (PF_CP) 0, 0, 0 }
 };
 /* *INDENT-ON* */
 
@@ -92,7 +98,7 @@ bi_funct_init(void)
     register const BI_REC *p;
     register SYMTAB *stp;
 
-    for (p = bi_funct; p->name; p++) {
+    for (p = bi_funct; p->name[0]; p++) {
 	stp = insert(p->name);
 	stp->type = ST_BUILTIN;
 	stp->stval.bip = p;
@@ -120,7 +126,7 @@ bi_length(CELL *sp)
 {
     size_t len;
 
-    TRACE_FUNC("bi_length", sp);
+    TRACE_FUNC2("bi_length", sp, 1);
 
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
@@ -137,7 +143,7 @@ bi_length(CELL *sp)
 CELL *
 bi_alength(CELL *sp)
 {
-    TRACE_FUNC("bi_alength", sp);
+    TRACE_FUNC2("bi_alength", sp, 1);
 
     sp->type = C_DOUBLE;
     sp->dval = (double) ((ARRAY) sp->ptr)->size;
@@ -195,7 +201,7 @@ bi_index(CELL *sp)
     size_t idx;
     size_t len;
 
-    TRACE_FUNC("bi_index", sp);
+    TRACE_FUNC2("bi_index", sp, 2);
 
     sp--;
     if (TEST2(sp) != TWO_STRINGS)
@@ -309,7 +315,7 @@ bi_match(CELL *sp)
     char *p;
     size_t length;
 
-    TRACE_FUNC("bi_match", sp);
+    TRACE_FUNC2("bi_match", sp + 1, 2);
 
     if (sp->type != C_RE)
 	cast_to_RE(sp);
@@ -351,7 +357,7 @@ bi_to##case(CELL *sp) \
     size_t len; \
     register char *p, *q; \
 \
-    TRACE_FUNC("bi_to" #case, sp); \
+    TRACE_FUNC2("bi_to" #case, sp, 1); \
 \
     if (sp->type != C_STRING) \
         cast1_to_s(sp); \
@@ -366,9 +372,11 @@ bi_to##case(CELL *sp) \
     free_STRING(old); \
     return_CELL("bi_to" #case, sp); \
 }
+/* *INDENT-OFF* */
 BI_TOCASE(upper)
 BI_TOCASE(lower)
 #undef BI_TOCASE
+/* *INDENT-ON* */
 
 /*
  * Like gawk...
@@ -379,7 +387,7 @@ bi_systime(CELL *sp)
     time_t result;
     time(&result);
 
-    TRACE_FUNC("bi_systime", sp);
+    TRACE_FUNC2("bi_systime", sp, 0);
 
     sp++;
     sp->type = C_DOUBLE;
@@ -400,7 +408,7 @@ bi_mktime(CELL *sp)
     struct tm my_tm;
     STRING *sval = string(sp);
 
-    TRACE_FUNC("bi_mktime", sp);
+    TRACE_FUNC2("bi_mktime", sp, 1);
 
     if (!sval)
 	goto error;
@@ -543,7 +551,7 @@ fplib_err(
 CELL *
 bi_sin(CELL *sp)
 {
-    TRACE_FUNC("bi_sin", sp);
+    TRACE_FUNC2("bi_sin", sp, 1);
 
 #if ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -568,7 +576,7 @@ bi_sin(CELL *sp)
 CELL *
 bi_cos(CELL *sp)
 {
-    TRACE_FUNC("bi_cos", sp);
+    TRACE_FUNC2("bi_cos", sp, 1);
 
 #if ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -593,7 +601,7 @@ bi_cos(CELL *sp)
 CELL *
 bi_atan2(CELL *sp)
 {
-    TRACE_FUNC("bi_atan2", sp);
+    TRACE_FUNC2("bi_atan2", sp, 2);
 
 #if  !	STDC_MATHERR
     sp--;
@@ -617,7 +625,7 @@ bi_atan2(CELL *sp)
 CELL *
 bi_log(CELL *sp)
 {
-    TRACE_FUNC("bi_log", sp);
+    TRACE_FUNC2("bi_log", sp, 1);
 
 #if ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -642,7 +650,7 @@ bi_log(CELL *sp)
 CELL *
 bi_exp(CELL *sp)
 {
-    TRACE_FUNC("bi_exp", sp);
+    TRACE_FUNC2("bi_exp", sp, 1);
 
 #if  ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -668,7 +676,7 @@ bi_exp(CELL *sp)
 CELL *
 bi_int(CELL *sp)
 {
-    TRACE_FUNC("bi_int", sp);
+    TRACE_FUNC2("bi_int", sp, 1);
 
     if (sp->type != C_DOUBLE)
 	cast1_to_d(sp);
@@ -679,7 +687,7 @@ bi_int(CELL *sp)
 CELL *
 bi_sqrt(CELL *sp)
 {
-    TRACE_FUNC("bi_sqrt", sp);
+    TRACE_FUNC2("bi_sqrt", sp, 1);
 
 #if  ! STDC_MATHERR
     if (sp->type != C_DOUBLE)
@@ -765,16 +773,16 @@ CELL *
 bi_srand(CELL *sp)
 {
 #ifdef USE_SYSTEM_SRAND
-    static long seed = 1;
     static CELL cseed =
     {
 	C_DOUBLE, 0, 0, 1.0
     };
+    double seed32;
 #endif
 
     CELL c;
 
-    TRACE_FUNC("bi_srand", sp);
+    TRACE_FUNC2("bi_srand", sp, (sp->type != C_NOINIT));
 
     if (sp->type == C_NOINIT)	/* seed off clock */
     {
@@ -793,8 +801,8 @@ bi_srand(CELL *sp)
     }
 
 #ifdef USE_SYSTEM_SRAND
-    seed = d_to_l(cseed.dval);
-    mawk_srand((unsigned) seed);
+    seed32 = fmod(cseed.dval, (double) Max_UInt);
+    mawk_srand((unsigned) seed32);
 #else
     /* The old seed is now in *sp ; move the value in cseed to
        seed in range [1,M) */
@@ -822,7 +830,7 @@ bi_srand(CELL *sp)
 CELL *
 bi_rand(CELL *sp)
 {
-    TRACE_FUNC("bi_rand", sp);
+    TRACE_FUNC2("bi_rand", sp, 0);
 
 #ifdef USE_SYSTEM_SRAND
     {
@@ -859,7 +867,7 @@ bi_close(CELL *sp)
 {
     int x;
 
-    TRACE_FUNC("bi_close", sp);
+    TRACE_FUNC2("bi_close", sp, 1);
 
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
@@ -876,9 +884,9 @@ bi_fflush(CELL *sp)
 {
     int ret = 0;
 
-    TRACE_FUNC("bi_fflush", sp);
+    TRACE_FUNC2("bi_fflush", sp, (sp->type != C_NOINIT));
 
-    if (sp->type == 0)
+    if (sp->type == C_NOINIT)
 	fflush(stdout);
     else {
 	sp--;
@@ -899,7 +907,7 @@ bi_system(CELL *sp GCC_UNUSED)
 {
     int ret_val;
 
-    TRACE_FUNC("bi_system", sp);
+    TRACE_FUNC2("bi_system", sp + 1, 1);
 
     if (sp->type < C_STRING)
 	cast1_to_s(sp);
@@ -943,7 +951,7 @@ bi_getline(CELL *sp)
     TRACE_FUNC("bi_getline", sp);
 
     switch (sp->type) {
-    case 0:
+    case C_NOINIT:
 	sp--;
 	if (!main_fin)
 	    open_main();
