@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: init.h,v 1.8 2024/08/18 19:44:27 tom Exp $
+ * $MawkId: init.h,v 1.9 2024/08/25 17:21:52 tom Exp $
  */
 
 /* init.h  */
@@ -19,15 +19,19 @@ the GNU General Public License, version 2, 1991.
 #ifndef  INIT_H
 #define  INIT_H
 
-#include "symtype.h"
+#include <symtype.h>
 
 /* nodes to link file names for multiple
    -f option */
 
-typedef struct pfile {
-    struct pfile *link;
+typedef struct _pfile
+#ifdef Visible_PFILE
+{
+    struct _pfile *link;
     char *fname;
-} PFILE;
+}
+#endif
+PFILE;
 
 extern PFILE *pfile_list;
 
@@ -47,7 +51,7 @@ void fpe_init(void);
 void load_environ(ARRAY);
 void set_stdio(void);
 
-GCC_NORETURN void print_version(FILE *fp);
+ GCC_NORETURN void print_version(FILE *fp);
 int is_cmdline_assign(char *);
 
 #endif /* INIT_H  */
