@@ -1,3 +1,6 @@
+#ifndef _yy_defines_h_
+#define _yy_defines_h_
+
 #define UNEXPECTED 257
 #define BAD_DECIMAL 258
 #define NL 259
@@ -79,12 +82,12 @@
 #endif
 #ifndef YYSTYPE_IS_DECLARED
 #define YYSTYPE_IS_DECLARED 1
-typedef union{
+typedef union YYSTYPE{
   CELL     *cp ;
   SYMTAB   *stp ;
   int      start ;   /* code starting address as offset from code_base */
   PF_CP    fp ;      /* ptr to a (print/printf) or (sub/gsub) function */
-  BI_REC   *bip ;    /* ptr to info about a builtin */
+  const BI_REC *bip ; /* ptr to info about a builtin */
   FBLOCK   *fbp  ;   /* ptr to a function block */
   ARG2_REC *arg2p ;
   CA_REC   *ca_p  ;
@@ -93,3 +96,5 @@ typedef union{
 } YYSTYPE;
 #endif /* !YYSTYPE_IS_DECLARED */
 extern YYSTYPE yylval;
+
+#endif /* _yy_defines_h_ */
